# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: protocol/packagewrapper.proto

from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='protocol/packagewrapper.proto',
  package='xhhk.protocol.packagewrapper',
  syntax='proto3',
  serialized_options=b'\n%com.xhhk.protocol.packagewrapperprotoB\032XhhkPackageWrapperProtocolZ\025.;packagewrapperproto\252\002!xhhk.protocol.packagewrapperproto',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x1dprotocol/packagewrapper.proto\x12\x1cxhhk.protocol.packagewrapper\"\xca\x01\n\x15PackageWrapperRequest\x12\x0b\n\x03sid\x18\x01 \x01(\r\x12\x0b\n\x03seq\x18\x02 \x01(\r\x12\x0b\n\x03uid\x18\x03 \x01(\t\x12\x10\n\x08\x63\x61tegory\x18\x04 \x01(\t\x12\x0f\n\x07routing\x18\x05 \x01(\t\x12\r\n\x05token\x18\x06 \x01(\t\x12\x10\n\x08personid\x18\x07 \x01(\t\x12\x11\n\tuseragent\x18\x08 \x01(\t\x12\x0f\n\x07\x62izcode\x18\t \x01(\t\x12\x11\n\trequestid\x18\n \x01(\t\x12\x0f\n\x07payload\x18\x0b \x01(\x0c\"\xbf\x01\n\x16PackageWrapperResponse\x12>\n\x04mode\x18\x01 \x01(\x0e\x32\x30.xhhk.protocol.packagewrapper.PackageWrapperMode\x12\x0b\n\x03uid\x18\x02 \x03(\t\x12\x10\n\x08personid\x18\x03 \x03(\t\x12\x11\n\tuseragent\x18\x04 \x01(\t\x12\x0f\n\x07\x62izcode\x18\x05 \x01(\t\x12\x11\n\trequestid\x18\x06 \x01(\t\x12\x0f\n\x07payload\x18\x07 \x01(\x0c\"\xe5\x01\n\x18\x43lientLoginActionRequest\x12=\n\x06\x61\x63tion\x18\x01 \x01(\x0e\x32-.xhhk.protocol.packagewrapper.LoginActionMode\x12\x0b\n\x03sid\x18\x02 \x01(\t\x12\x0b\n\x03seq\x18\x03 \x01(\r\x12\x0b\n\x03uid\x18\x04 \x01(\t\x12\x10\n\x08serverid\x18\x05 \x01(\t\x12\x0c\n\x04name\x18\x06 \x01(\t\x12\x10\n\x08personid\x18\x07 \x01(\t\x12\r\n\x05token\x18\x08 \x01(\t\x12\x0f\n\x07payload\x18\t \x01(\x0c\x12\x11\n\tuseragent\x18\n \x01(\t\"\x87\x01\n\x15GetOnlineUsersRequest\x12\x0b\n\x03seq\x18\x01 \x01(\r\x12=\n\x04mode\x18\x02 \x01(\x0e\x32/.xhhk.protocol.packagewrapper.GetOnlineUserMode\x12\x0f\n\x07payload\x18\x03 \x01(\x0c\x12\x11\n\tuseragent\x18\x04 \x01(\t\"(\n\x07UidInfo\x12\x0b\n\x03uid\x18\x01 \x01(\t\x12\x10\n\x08personid\x18\x02 \x01(\t\"\xa8\x01\n\x16GetOnlineUsersResponse\x12\x0b\n\x03seq\x18\x01 \x01(\r\x12=\n\x04mode\x18\x02 \x01(\x0e\x32/.xhhk.protocol.packagewrapper.GetOnlineUserMode\x12\r\n\x05\x63ount\x18\x03 \x01(\r\x12\x33\n\x04uids\x18\x04 \x03(\x0b\x32%.xhhk.protocol.packagewrapper.UidInfo*?\n\x12PackageWrapperMode\x12\x0b\n\x07Unicast\x10\x00\x12\r\n\tMulticast\x10\x01\x12\r\n\tBroadcast\x10\x02*A\n\x0fLoginActionMode\x12\t\n\x05Login\x10\x00\x12\n\n\x06Logout\x10\x01\x12\n\n\x06\x42roken\x10\x03\x12\x0b\n\x07Kickout\x10\x04*(\n\x11GetOnlineUserMode\x12\t\n\x05\x43ount\x10\x00\x12\x08\n\x04List\x10\x01\x42~\n%com.xhhk.protocol.packagewrapperprotoB\x1aXhhkPackageWrapperProtocolZ\x15.;packagewrapperproto\xaa\x02!xhhk.protocol.packagewrapperprotob\x06proto3'
)

_PACKAGEWRAPPERMODE = _descriptor.EnumDescriptor(
  name='PackageWrapperMode',
  full_name='xhhk.protocol.packagewrapper.PackageWrapperMode',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='Unicast', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='Multicast', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='Broadcast', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=1045,
  serialized_end=1108,
)
_sym_db.RegisterEnumDescriptor(_PACKAGEWRAPPERMODE)

PackageWrapperMode = enum_type_wrapper.EnumTypeWrapper(_PACKAGEWRAPPERMODE)
_LOGINACTIONMODE = _descriptor.EnumDescriptor(
  name='LoginActionMode',
  full_name='xhhk.protocol.packagewrapper.LoginActionMode',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='Login', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='Logout', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='Broken', index=2, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='Kickout', index=3, number=4,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=1110,
  serialized_end=1175,
)
_sym_db.RegisterEnumDescriptor(_LOGINACTIONMODE)

LoginActionMode = enum_type_wrapper.EnumTypeWrapper(_LOGINACTIONMODE)
_GETONLINEUSERMODE = _descriptor.EnumDescriptor(
  name='GetOnlineUserMode',
  full_name='xhhk.protocol.packagewrapper.GetOnlineUserMode',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='Count', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='List', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=1177,
  serialized_end=1217,
)
_sym_db.RegisterEnumDescriptor(_GETONLINEUSERMODE)

GetOnlineUserMode = enum_type_wrapper.EnumTypeWrapper(_GETONLINEUSERMODE)
Unicast = 0
Multicast = 1
Broadcast = 2
Login = 0
Logout = 1
Broken = 3
Kickout = 4
Count = 0
List = 1



_PACKAGEWRAPPERREQUEST = _descriptor.Descriptor(
  name='PackageWrapperRequest',
  full_name='xhhk.protocol.packagewrapper.PackageWrapperRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='sid', full_name='xhhk.protocol.packagewrapper.PackageWrapperRequest.sid', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='seq', full_name='xhhk.protocol.packagewrapper.PackageWrapperRequest.seq', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='uid', full_name='xhhk.protocol.packagewrapper.PackageWrapperRequest.uid', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='category', full_name='xhhk.protocol.packagewrapper.PackageWrapperRequest.category', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='routing', full_name='xhhk.protocol.packagewrapper.PackageWrapperRequest.routing', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='token', full_name='xhhk.protocol.packagewrapper.PackageWrapperRequest.token', index=5,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='personid', full_name='xhhk.protocol.packagewrapper.PackageWrapperRequest.personid', index=6,
      number=7, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='useragent', full_name='xhhk.protocol.packagewrapper.PackageWrapperRequest.useragent', index=7,
      number=8, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='bizcode', full_name='xhhk.protocol.packagewrapper.PackageWrapperRequest.bizcode', index=8,
      number=9, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='requestid', full_name='xhhk.protocol.packagewrapper.PackageWrapperRequest.requestid', index=9,
      number=10, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='payload', full_name='xhhk.protocol.packagewrapper.PackageWrapperRequest.payload', index=10,
      number=11, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=64,
  serialized_end=266,
)


_PACKAGEWRAPPERRESPONSE = _descriptor.Descriptor(
  name='PackageWrapperResponse',
  full_name='xhhk.protocol.packagewrapper.PackageWrapperResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='mode', full_name='xhhk.protocol.packagewrapper.PackageWrapperResponse.mode', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='uid', full_name='xhhk.protocol.packagewrapper.PackageWrapperResponse.uid', index=1,
      number=2, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='personid', full_name='xhhk.protocol.packagewrapper.PackageWrapperResponse.personid', index=2,
      number=3, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='useragent', full_name='xhhk.protocol.packagewrapper.PackageWrapperResponse.useragent', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='bizcode', full_name='xhhk.protocol.packagewrapper.PackageWrapperResponse.bizcode', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='requestid', full_name='xhhk.protocol.packagewrapper.PackageWrapperResponse.requestid', index=5,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='payload', full_name='xhhk.protocol.packagewrapper.PackageWrapperResponse.payload', index=6,
      number=7, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=269,
  serialized_end=460,
)


_CLIENTLOGINACTIONREQUEST = _descriptor.Descriptor(
  name='ClientLoginActionRequest',
  full_name='xhhk.protocol.packagewrapper.ClientLoginActionRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='action', full_name='xhhk.protocol.packagewrapper.ClientLoginActionRequest.action', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='sid', full_name='xhhk.protocol.packagewrapper.ClientLoginActionRequest.sid', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='seq', full_name='xhhk.protocol.packagewrapper.ClientLoginActionRequest.seq', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='uid', full_name='xhhk.protocol.packagewrapper.ClientLoginActionRequest.uid', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='serverid', full_name='xhhk.protocol.packagewrapper.ClientLoginActionRequest.serverid', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='name', full_name='xhhk.protocol.packagewrapper.ClientLoginActionRequest.name', index=5,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='personid', full_name='xhhk.protocol.packagewrapper.ClientLoginActionRequest.personid', index=6,
      number=7, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='token', full_name='xhhk.protocol.packagewrapper.ClientLoginActionRequest.token', index=7,
      number=8, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='payload', full_name='xhhk.protocol.packagewrapper.ClientLoginActionRequest.payload', index=8,
      number=9, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='useragent', full_name='xhhk.protocol.packagewrapper.ClientLoginActionRequest.useragent', index=9,
      number=10, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=463,
  serialized_end=692,
)


_GETONLINEUSERSREQUEST = _descriptor.Descriptor(
  name='GetOnlineUsersRequest',
  full_name='xhhk.protocol.packagewrapper.GetOnlineUsersRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='seq', full_name='xhhk.protocol.packagewrapper.GetOnlineUsersRequest.seq', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='mode', full_name='xhhk.protocol.packagewrapper.GetOnlineUsersRequest.mode', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='payload', full_name='xhhk.protocol.packagewrapper.GetOnlineUsersRequest.payload', index=2,
      number=3, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='useragent', full_name='xhhk.protocol.packagewrapper.GetOnlineUsersRequest.useragent', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=695,
  serialized_end=830,
)


_UIDINFO = _descriptor.Descriptor(
  name='UidInfo',
  full_name='xhhk.protocol.packagewrapper.UidInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='uid', full_name='xhhk.protocol.packagewrapper.UidInfo.uid', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='personid', full_name='xhhk.protocol.packagewrapper.UidInfo.personid', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=832,
  serialized_end=872,
)


_GETONLINEUSERSRESPONSE = _descriptor.Descriptor(
  name='GetOnlineUsersResponse',
  full_name='xhhk.protocol.packagewrapper.GetOnlineUsersResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='seq', full_name='xhhk.protocol.packagewrapper.GetOnlineUsersResponse.seq', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='mode', full_name='xhhk.protocol.packagewrapper.GetOnlineUsersResponse.mode', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='count', full_name='xhhk.protocol.packagewrapper.GetOnlineUsersResponse.count', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='uids', full_name='xhhk.protocol.packagewrapper.GetOnlineUsersResponse.uids', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=875,
  serialized_end=1043,
)

_PACKAGEWRAPPERRESPONSE.fields_by_name['mode'].enum_type = _PACKAGEWRAPPERMODE
_CLIENTLOGINACTIONREQUEST.fields_by_name['action'].enum_type = _LOGINACTIONMODE
_GETONLINEUSERSREQUEST.fields_by_name['mode'].enum_type = _GETONLINEUSERMODE
_GETONLINEUSERSRESPONSE.fields_by_name['mode'].enum_type = _GETONLINEUSERMODE
_GETONLINEUSERSRESPONSE.fields_by_name['uids'].message_type = _UIDINFO
DESCRIPTOR.message_types_by_name['PackageWrapperRequest'] = _PACKAGEWRAPPERREQUEST
DESCRIPTOR.message_types_by_name['PackageWrapperResponse'] = _PACKAGEWRAPPERRESPONSE
DESCRIPTOR.message_types_by_name['ClientLoginActionRequest'] = _CLIENTLOGINACTIONREQUEST
DESCRIPTOR.message_types_by_name['GetOnlineUsersRequest'] = _GETONLINEUSERSREQUEST
DESCRIPTOR.message_types_by_name['UidInfo'] = _UIDINFO
DESCRIPTOR.message_types_by_name['GetOnlineUsersResponse'] = _GETONLINEUSERSRESPONSE
DESCRIPTOR.enum_types_by_name['PackageWrapperMode'] = _PACKAGEWRAPPERMODE
DESCRIPTOR.enum_types_by_name['LoginActionMode'] = _LOGINACTIONMODE
DESCRIPTOR.enum_types_by_name['GetOnlineUserMode'] = _GETONLINEUSERMODE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

PackageWrapperRequest = _reflection.GeneratedProtocolMessageType('PackageWrapperRequest', (_message.Message,), {
  'DESCRIPTOR' : _PACKAGEWRAPPERREQUEST,
  '__module__' : 'protocol.packagewrapper_pb2'
  # @@protoc_insertion_point(class_scope:xhhk.protocol.packagewrapper.PackageWrapperRequest)
  })
_sym_db.RegisterMessage(PackageWrapperRequest)

PackageWrapperResponse = _reflection.GeneratedProtocolMessageType('PackageWrapperResponse', (_message.Message,), {
  'DESCRIPTOR' : _PACKAGEWRAPPERRESPONSE,
  '__module__' : 'protocol.packagewrapper_pb2'
  # @@protoc_insertion_point(class_scope:xhhk.protocol.packagewrapper.PackageWrapperResponse)
  })
_sym_db.RegisterMessage(PackageWrapperResponse)

ClientLoginActionRequest = _reflection.GeneratedProtocolMessageType('ClientLoginActionRequest', (_message.Message,), {
  'DESCRIPTOR' : _CLIENTLOGINACTIONREQUEST,
  '__module__' : 'protocol.packagewrapper_pb2'
  # @@protoc_insertion_point(class_scope:xhhk.protocol.packagewrapper.ClientLoginActionRequest)
  })
_sym_db.RegisterMessage(ClientLoginActionRequest)

GetOnlineUsersRequest = _reflection.GeneratedProtocolMessageType('GetOnlineUsersRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETONLINEUSERSREQUEST,
  '__module__' : 'protocol.packagewrapper_pb2'
  # @@protoc_insertion_point(class_scope:xhhk.protocol.packagewrapper.GetOnlineUsersRequest)
  })
_sym_db.RegisterMessage(GetOnlineUsersRequest)

UidInfo = _reflection.GeneratedProtocolMessageType('UidInfo', (_message.Message,), {
  'DESCRIPTOR' : _UIDINFO,
  '__module__' : 'protocol.packagewrapper_pb2'
  # @@protoc_insertion_point(class_scope:xhhk.protocol.packagewrapper.UidInfo)
  })
_sym_db.RegisterMessage(UidInfo)

GetOnlineUsersResponse = _reflection.GeneratedProtocolMessageType('GetOnlineUsersResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETONLINEUSERSRESPONSE,
  '__module__' : 'protocol.packagewrapper_pb2'
  # @@protoc_insertion_point(class_scope:xhhk.protocol.packagewrapper.GetOnlineUsersResponse)
  })
_sym_db.RegisterMessage(GetOnlineUsersResponse)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)
